@@ -55,19 +55,24 @@ class RenderBase(_ValidatingResource):
         pool_d.addCallback(self._writeOutput, request)
         pool_d.addErrback(self._timeoutError, request)
         pool_d.addErrback(self._renderError, request)
+        pool_d.addErrback(self._badRequest, request)
         pool_d.addErrback(self._internalError, request)
         pool_d.addBoth(self._finishRequest, request)
         request.starttime = time.time()
         return NOT_DONE_YET
 
     def render_POST(self, request):
-        # this check is required only in request not coming from the splash proxy service.
-        if not self.is_proxy_request:
-            content_type = request.getHeader('content-type')
-            if content_type != 'application/javascript':
-                request.setResponseCode(415)
-                request.write("Request content-type not supported\n")
-                return
+        if self.is_proxy_request:
+            # If request comes from splash proxy service don't handle
+            # special content-types.
+            # TODO: pass http method to WebpageRender explicitly.
+            return self.render_GET(request)
+
+        content_type = request.getHeader('content-type')
+        if not any(ct in content_type for ct in ['application/javascript', 'application/json']):
+            request.setResponseCode(415)
+            request.write("Request content-type not supported\n")
+            return
 
         return self.render_GET(request)
 
@@ -111,6 +116,11 @@ class RenderBase(_ValidatingResource):
         log.err()
         sentry.capture(failure)
 
+    def _badRequest(self, failure, request):
+        failure.trap(BadRequest)
+        request.setResponseCode(400)
+        request.write(str(failure.value) + "\n")
+
     def _finishRequest(self, _, request):
         if not request._disconnected:
             request.finish()
@@ -155,15 +165,47 @@ def _check_filters(pool, request):
 
 
 def _get_javascript_params(request, js_profiles_path):
-    js_profile = _check_js_profile(request, js_profiles_path, getarg(request, 'js', None))
+    return dict(
+        js_profile=_check_js_profile(request, js_profiles_path, getarg(request, 'js', None)),
+        js_source=_get_js_source(request),
+    )
+
+def _get_headers_params(request):
+    headers = None
+
+    if getattr(request, 'inspect_me', False):
+        # use headers from splash_request
+        headers = [
+            (name, value)
+            for name, values in request.requestHeaders.getAllRawHeaders()
+            for value in values
+        ]
+
+    headers = getarg(request, "headers", default=headers, type=None)
+    if headers is None:
+        return headers
+
+    if not isinstance(headers, (list, tuple, dict)):
+        raise BadRequest("'headers' must be either JSON array of (name, value) pairs or JSON object")
+
+    if isinstance(headers, (list, tuple)):
+        for el in headers:
+            if not (isinstance(el, (list, tuple)) and len(el) == 2 and all(isinstance(e, basestring) for e in el)):
+                raise BadRequest("'headers' must be either JSON array of (name, value) pairs or JSON object")
+
+    return headers
+
+
+def _get_js_source(request):
     js_source = getarg(request, 'js_source', None)
     if js_source is not None:
-        return js_source, js_profile
+        return js_source
 
+    # handle application/javascript POST requests
     if request.method == 'POST':
-        return request.content.getvalue(), js_profile
-    else:
-        return None, js_profile
+        content_type = request.getHeader('Content-Type')
+        if content_type and 'application/javascript' in content_type:
+            return request.content.read()
 
 
 def _check_js_profile(request, js_profiles_path, js_profile):
@@ -179,25 +221,39 @@ def _check_js_profile(request, js_profiles_path, js_profile):
         return profile_dir
 
 
-def _get_png_params(request, js_profiles_path):
-    url, baseurl, wait_time, viewport, js_source, js_profile, images = _get_common_params(request, js_profiles_path)
-    width = getarg(request, "width", None, type=int, range=(1, defaults.MAX_WIDTH))
-    height = getarg(request, "height", None, type=int, range=(1, defaults.MAX_HEIGTH))
-    return url, baseurl, wait_time, viewport, js_source, js_profile, images, width, height
+def _get_png_params(request):
+    return dict(
+        width = getarg(request, "width", None, type=int, range=(1, defaults.MAX_WIDTH)),
+        height = getarg(request, "height", None, type=int, range=(1, defaults.MAX_HEIGTH)),
+    )
 
 
 def _get_common_params(request, js_profiles_path):
-    url = getarg(request, "url")
-    baseurl = getarg(request, "baseurl", None)
+    """ Return arguments common for all endpoints """
     wait_time = getarg(request, "wait", defaults.WAIT_TIME, type=float, range=(0, defaults.MAX_WAIT_TIME))
-    js_source, js_profile = _get_javascript_params(request, js_profiles_path)
-    images = getarg_bool(request, "images", defaults.AUTOLOAD_IMAGES)
-
     viewport = getarg(request, "viewport", defaults.VIEWPORT)
     _check_viewport(viewport, wait_time, defaults.VIEWPORT_MAX_WIDTH,
                     defaults.VIEWPORT_MAX_HEIGTH, defaults.VIEWPORT_MAX_AREA)
 
-    return url, baseurl, wait_time, viewport, js_source, js_profile, images
+    url = getarg(request, "url", type=None)
+    baseurl = getarg(request, "baseurl", default=None, type=None)
+    if isinstance(url, unicode):
+        url = url.encode('utf8')
+    if isinstance(baseurl, unicode):
+        baseurl = baseurl.encode('utf8')
+
+    res = dict(
+        url = url,
+        baseurl = baseurl,
+        wait = wait_time,
+        viewport = viewport,
+        images = getarg_bool(request, "images", defaults.AUTOLOAD_IMAGES),
+        headers = _get_headers_params(request),
+
+        proxy = getarg(request, "proxy", None),
+    )
+    res.update(_get_javascript_params(request, js_profiles_path))
+    return res
 
 
 class RenderHtml(RenderBase):
@@ -206,7 +262,7 @@ class RenderHtml(RenderBase):
 
     def _getRender(self, request):
         params = _get_common_params(request, self.js_profiles_path)
-        return self.pool.render(HtmlRender, request, *params)
+        return self.pool.render(HtmlRender, request, **params)
 
 
 class RenderPng(RenderBase):
@@ -214,8 +270,9 @@ class RenderPng(RenderBase):
     content_type = "image/png"
 
     def _getRender(self, request):
-        params = _get_png_params(request, self.js_profiles_path)
-        return self.pool.render(PngRender, request, *params)
+        params = _get_common_params(request, self.js_profiles_path)
+        params.update(_get_png_params(request))
+        return self.pool.render(PngRender, request, **params)
 
 
 class RenderJson(RenderBase):
@@ -223,20 +280,19 @@ class RenderJson(RenderBase):
     content_type = "application/json"
 
     def _getRender(self, request):
-        url, baseurl, wait_time, viewport, js_source, js_profile, images, width, height = _get_png_params(request, self.js_profiles_path)
+        params = _get_common_params(request, self.js_profiles_path)
+        params.update(_get_png_params(request))
+        params.update(
+            html = getarg_bool(request, "html", defaults.DO_HTML),
+            iframes = getarg_bool(request, "iframes", defaults.DO_IFRAMES),
+            png = getarg_bool(request, "png", defaults.DO_PNG),
+            script = getarg_bool(request, "script", defaults.SHOW_SCRIPT),
+            console = getarg_bool(request, "console", defaults.SHOW_CONSOLE),
+            history = getarg_bool(request, "history", defaults.SHOW_HISTORY),
+            har = getarg_bool(request, "har", defaults.SHOW_HAR),
+        )
+        return self.pool.render(JsonRender, request, **params)
 
-        html = getarg_bool(request, "html", defaults.DO_HTML)
-        iframes = getarg_bool(request, "iframes", defaults.DO_IFRAMES)
-        png = getarg_bool(request, "png", defaults.DO_PNG)
-        script = getarg_bool(request, "script", defaults.SHOW_SCRIPT)
-        console = getarg_bool(request, "console", defaults.SHOW_CONSOLE)
-        history = getarg_bool(request, "history", defaults.SHOW_HISTORY)
-        har = getarg_bool(request, "har", defaults.SHOW_HAR)
-
-        return self.pool.render(JsonRender, request,
-                                url, baseurl, wait_time, viewport, js_source, js_profile, images,
-                                html, iframes, png, script, console,
-                                width, height, history, har)
 
 class RenderHar(RenderBase):
 
@@ -244,7 +300,7 @@ class RenderHar(RenderBase):
 
     def _getRender(self, request):
         params = _get_common_params(request, self.js_profiles_path)
-        return self.pool.render(HarRender, request, *params)
+        return self.pool.render(HarRender, request, **params)
 
 
 class Debug(Resource):
@@ -281,24 +337,18 @@ class HarViewer(_ValidatingResource):
         return _get_common_params(request, self.pool.js_profiles_path)
 
     def render_GET(self, request):
-        url, baseurl, wait_time, viewport, js_source, js_profile, images = self._validate_params(request)
+        params = self._validate_params(request)
+        url = params['url']
         if not url.lower().startswith('http'):
             url = 'http://' + url
 
-        params = {
-            'url': url,
-            'baseurl': baseurl,
-            'wait': wait_time,
-            'viewport': viewport,
-            'js_source': js_source,
-            'js': js_profile,
-            'images': images,
+        params.update({
             'timeout': _get_timeout_arg(request),
 
             'har': 1,
             'png': 1,
             'html': 1,
-        }
+        })
         params = {k:v for k,v in params.items() if v is not None}
 
         request.addCookie('phaseInterval', 120000)  # disable "phases" HAR Viewer feature
@@ -445,7 +495,12 @@ class HarViewer(_ValidatingResource):
                 var viewer = event.target.repObject;
                 $("#status").text("Rendering, please wait..");
 
-                $.getJSON("/render.json", params).done(function(data){
+                $.ajax("/render.json", {
+                    "contentType": "application/json",
+                    "dataType": "json",
+                    "type": "POST",
+                    "data": JSON.stringify(params)
+                }).done(function(data){
                     var har = data['har'];
                     var png = data['png'];
                     var html = data['html'];
