@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
-import unittest, requests, json, base64, urllib
+import unittest
+import json
+import base64
+import urllib
 from functools import wraps
 from cStringIO import StringIO
+
+import pytest
+import requests
 from PIL import Image
-from splash.tests import ts
 from splash.tests.utils import NON_EXISTING_RESOLVABLE
 from splash.tests.utils import SplashServer
 
@@ -34,29 +39,34 @@ class DirectRequestHandler(object):
 
     render_format = "html"
 
+    def __init__(self, ts):
+        self.ts = ts
+
     @property
     def host(self):
-        return "localhost:%s" % ts.splashserver.portnum
+        return "localhost:%s" % self.ts.splashserver.portnum
 
     def request(self, query, render_format=None, headers=None):
-        render_format = render_format or self.render_format
-        if isinstance(query, dict):
-            url = "http://%s/render.%s" % (self.host, render_format)
-            return requests.get(url, params=query, headers=headers)
-        else:
-            url = "http://%s/render.%s?%s" % (self.host, render_format, query)
-            return requests.get(url, headers=headers)
+        url, params = self._url_and_params(render_format, query)
+        return requests.get(url, params=params, headers=headers)
 
     def post(self, query, render_format=None, payload=None, headers=None):
+        url, params = self._url_and_params(render_format, query)
+        return requests.post(url, params=params, data=payload, headers=headers)
+
+    def _url_and_params(self, render_format, query):
         render_format = render_format or self.render_format
         if isinstance(query, dict):
             url = "http://%s/render.%s" % (self.host, render_format)
-            return requests.post(url, params=query, data=payload, headers=headers)
+            params = query
         else:
             url = "http://%s/render.%s?%s" % (self.host, render_format, query)
-            return requests.post(url, data=payload, headers=headers)
+            params = None
+        return url, params
 
 
+@pytest.mark.usefixtures("class_ts")
+@pytest.mark.usefixtures("print_ts_output")
 class BaseRenderTest(unittest.TestCase):
 
     render_format = "html"
@@ -68,19 +78,13 @@ class BaseRenderTest(unittest.TestCase):
             try:
                 from twisted.web.server import GzipEncoderFactory
             except ImportError:
-                from nose import SkipTest
-                raise SkipTest("Gzip support is not available in old Twisted")
+                pytest.skip("Gzip support is not available in old Twisted")
 
     def mockurl(self, path, host='localhost'):
-        return ts.mockserver.url(path, self.use_gzip, host=host)
-
-    def tearDown(self):
-        # we must consume splash output because subprocess.PIPE is used
-        ts.print_output()
-        super(BaseRenderTest, self).tearDown()
+        return self.ts.mockserver.url(path, self.use_gzip, host=host)
 
     def _get_handler(self):
-        handler = self.request_handler()
+        handler = self.request_handler(self.ts)
         handler.render_format = self.render_format
         return handler
 
@@ -108,7 +112,7 @@ class _RenderTest(BaseRenderTest):
 
     @skip_proxy
     def test_missing_url(self):
-        r = self.request("")
+        r = self.request({})
         self.assertEqual(r.status_code, 400)
         self.assertTrue("url" in r.text)
 
@@ -144,7 +148,7 @@ class RenderHtmlTest(_RenderTest):
 
     @https_only
     def test_jsrender_https(self):
-        self._test_jsrender(ts.mockserver.https_url("jsrender"))
+        self._test_jsrender(self.ts.mockserver.https_url("jsrender"))
 
     def _test_jsrender(self, url):
         r = self.request({"url": url})
@@ -238,7 +242,7 @@ class RenderPngTest(_RenderTest):
 
     @https_only
     def test_ok_https(self):
-        self._test_ok(ts.mockserver.https_url("jsrender"))
+        self._test_ok(self.ts.mockserver.https_url("jsrender"))
 
     def _test_ok(self, url):
         r = self.request("url=%s" % url)
@@ -308,7 +312,7 @@ class RenderJsonTest(_RenderTest):
 
     @https_only
     def test_jsrender_https_html(self):
-        self.assertSameHtml(ts.mockserver.https_url("jsrender"))
+        self.assertSameHtml(self.ts.mockserver.https_url("jsrender"))
 
     def test_jsalert_html(self):
         self.assertSameHtml(self.mockurl("jsalert"), {'timeout': 3})
@@ -328,7 +332,7 @@ class RenderJsonTest(_RenderTest):
 
     @https_only
     def test_jsrender_https_png(self):
-        self.assertSamePng(ts.mockserver.https_url("jsrender"))
+        self.assertSamePng(self.ts.mockserver.https_url("jsrender"))
 
     def test_jsalert_png(self):
         self.assertSamePng(self.mockurl("jsalert"), {'timeout': 3})
@@ -357,7 +361,7 @@ class RenderJsonTest(_RenderTest):
 
     @https_only
     def test_fields_all(self):
-        query = {'url': ts.mockserver.https_url("iframes"),
+        query = {'url': self.ts.mockserver.https_url("iframes"),
                  "html": 1, "png": 1, "iframes": 1}
 
         res = self.request(query).json()
@@ -374,7 +378,7 @@ class RenderJsonTest(_RenderTest):
     @https_only
     def test_fields_no_html(self):
         # turn off returning HTML
-        query = {'url': ts.mockserver.https_url("iframes"),
+        query = {'url': self.ts.mockserver.https_url("iframes"),
                  'html': 0, 'png': 1, 'iframes': 1}
 
         res = self.request(query).json()
@@ -393,7 +397,7 @@ class RenderJsonTest(_RenderTest):
     @https_only
     def test_fields_no_screenshots(self):
         # turn off screenshots
-        query = {'url': ts.mockserver.https_url("iframes"),
+        query = {'url': self.ts.mockserver.https_url("iframes"),
                  'html': 1, 'png': 0, 'iframes': 1}
         res = self.request(query).json()
         self.assertFieldsInResponse(res, ["url", "requestedUrl", "childFrames",
@@ -402,7 +406,7 @@ class RenderJsonTest(_RenderTest):
 
     @https_only
     def test_fields_no_iframes(self):
-        query = {'url': ts.mockserver.https_url("iframes"),
+        query = {'url': self.ts.mockserver.https_url("iframes"),
                  'html': 1, 'png': 1, 'iframes': 0}
         res = self.request(query).json()
         self.assertFieldsInResponse(res, ["url", "requestedUrl", "geometry",
@@ -411,7 +415,7 @@ class RenderJsonTest(_RenderTest):
 
     @https_only
     def test_fields_default(self):
-        query = {'url': ts.mockserver.https_url("iframes")}
+        query = {'url': self.ts.mockserver.https_url("iframes")}
         res = self.request(query).json()
         self.assertFieldsInResponse(res, ["url", "requestedUrl", "geometry",
                                           "title"])
@@ -574,131 +578,25 @@ class IframesRenderTest(BaseRenderTest):
                    for child in result['childFrames'])
 
     def _iframes_request(self, params):
-        query = {'url': ts.mockserver.https_url("iframes"),
+        query = {'url': self.ts.mockserver.https_url("iframes"),
                  'iframes': 1, 'html': 1}
         query.update(params or {})
         return self.request(query).json()
 
 
-class RunJsTest(BaseRenderTest):
-    render_format = 'json'
-
-    CROSS_DOMAIN_JS = """
-    function getContents(){
-        var iframe = document.getElementById('external');
-        return iframe.contentDocument.getElementsByTagName('body')[0].innerHTML;
-    };
-    getContents();"""
-
-
-    def test_simple_js(self):
-        js_source = "function test(x){ return x; } test('abc');"
-        r = self._runjs_request(js_source).json()
-        self.assertEqual(r['script'], "abc")
-
-    def test_js_and_console(self):
-        js_source = """function test(x){ return x; }
-console.log('some log');
-console.log('another log');
-test('abc');"""
-        params = {'console': '1'}
-        r = self._runjs_request(js_source, params=params).json()
-        self.assertEqual(r['script'], "abc")
-        self.assertEqual(r['console'], ["some log", "another log"])
-
-    def test_js_modify_html(self):
-        js_source = """function test(x){ document.getElementById("p1").innerHTML=x; }
-test('Changed');"""
-        params = {'url': self.mockurl("jsrender")}
-        r = self._runjs_request(js_source, render_format='html', params=params)
-        self.assertTrue("Before" not in r.text)
-        self.assertTrue("Changed" in r.text)
-
-    def test_js_profile(self):
-        js_source = """test('abc');"""
-        params = {'url': self.mockurl("jsrender"), 'js' : 'test'}
-        r = self._runjs_request(js_source, params=params).json()
-        self.assertEqual(r['script'], "abc")
-
-    def test_js_profile_another_lib(self):
-        js_source = """test2('abc');"""
-        params = {'url': self.mockurl("jsrender"), 'js' : 'test'}
-        r = self._runjs_request(js_source, params=params).json()
-        self.assertEqual(r['script'], "abcabc")
-
-    def test_js_utf8_lib(self):
-        js_source = """console.log(test_utf8('abc')); test_utf8('abc');"""
-        params = {'url': self.mockurl("jsrender"), 'js' : 'test', 'console': '1'}
-        r = self._runjs_request(js_source, params=params).json()
-        self.assertEqual(r['script'], u'abc\xae')
-        self.assertEqual(r['console'], [u'abc\xae'])
-
-    def test_js_external_iframe(self):
-        # by default, cross-domain access is disabled, so this does nothing
-        params = {'url': self.mockurl("externaliframe")}
-        r = self._runjs_request(self.CROSS_DOMAIN_JS, params=params).json()
-        self.assertNotIn('script', r)
-
-    @skip_proxy
-    def test_js_external_iframe_cross_domain_enabled(self):
-        # cross-domain access should work if we enable it
-        with SplashServer(extra_args=['--js-cross-domain-access']) as splash:
-            query = {'url': self.mockurl("externaliframe"), 'script': 1}
-            headers = {'content-type': 'application/javascript'}
-            response = requests.post(
-                splash.url("render.json"),
-                params=query,
-                headers=headers,
-                data=self.CROSS_DOMAIN_JS,
-            )
-            self.assertEqual(response.json()['script'], u'EXTERNAL\n\n')
-
-    @skip_proxy
-    def test_js_incorrect_content_type(self):
-        js_source = "function test(x){ return x; } test('abc');"
-        headers = {'content-type': 'text/plain'}
-        r = self._runjs_request(js_source, headers=headers)
-        self.assertEqual(r.status_code, 415)
-
-    def test_proper_viewport(self):
-        js_source = """
-            function size() {
-                return [window.innerWidth, window.innerHeight].toString();
-            }
-            size();
-            """
-        params = {'viewport': '123x234'}
-        r = self._runjs_request(js_source, params=params).json()
-        self.assertEqual(r['script'], '123,234')
-
-    def test_js_invalid_profile(self):
-        js_source = """test('abc');"""
-        params = {'url': self.mockurl("jsrender"), 'js' : 'not_a_profile'}
-        r = self._runjs_request(js_source, params=params)
-        self.assertEqual(r.status_code, 400)
-
-    def _runjs_request(self, js_source, render_format=None, params=None, headers=None):
-        query = {'url': self.mockurl("jsrender"), 'script': 1}
-        query.update(params or {})
-        req_headers = {'content-type': 'application/javascript'}
-        req_headers.update(headers or {})
-        return self.post(query, render_format=render_format,
-                         payload=js_source, headers=req_headers)
-
 
+@pytest.mark.usefixtures("class_ts")
+@pytest.mark.usefixtures("print_ts_output")
 class TestTestSetup(unittest.TestCase):
-    def tearDown(self):
-        # we must consume splash output because subprocess.PIPE is used
-        ts.print_output()
 
     def test_mockserver_works(self):
-        r = requests.get(ts.mockserver.url("jsrender", gzip=False))
+        r = requests.get(self.ts.mockserver.url("jsrender", gzip=False))
         self.assertEqual(r.status_code, 200)
 
     def test_mockserver_https_works(self):
-        r = requests.get(ts.mockserver.https_url("jsrender"), verify=False)
+        r = requests.get(self.ts.mockserver.https_url("jsrender"), verify=False)
         self.assertEqual(r.status_code, 200)
 
     def test_splashserver_works(self):
-        r = requests.get('http://localhost:%s/debug' % ts.splashserver.portnum)
+        r = requests.get('http://localhost:%s/debug' % self.ts.splashserver.portnum)
         self.assertEqual(r.status_code, 200)
